#include <gccore.h>
#include <ogcsys.h>
#include <ogc/ipc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <zlib.h>

#include "gb.h"
#include "menu.h"
#include "filer.h"
#include "screen.h"
#include "renderer.h"
#include "pad.h"
#include "unzip.h"
#include "saveload.h"
#include "integer.h"

SETTING setting;
char oldpath[MAX_PATH] = "/EMULATOR/RIN/ROMS/"; //{ 0 };
char RomPath[MAX_PATH];
char RomName[MAX_PATH];
char SavePath[MAX_PATH];

const char *gbtype_names[] = {
	"AUTO",
	"GB",
	"SGB",
	"GBC",
	"GBA",
};

#ifdef DEBUG
int load_rom()
{
	extern int rom_gb_size;
	extern unsigned char rom_gb[];
	int romsize, ramsize;
	
	gb_init();
	
	romsize = rom_gb_size;
	memcpy(rom_image, rom_gb, romsize);
	ramsize = 0;
	memset(sram_space, 0, sizeof(sram_space));
	
	if (!gb_load_rom(rom_image, romsize, sram_space, ramsize))
		return -1;
	
	return 1;
}
#else
int load_rom(const char * Filename)
{
	FILE *fp;
	WORD bytes_read;
	char tmp[MAX_PATH];
	int romsize, ramsize;
	long fileSize;
	
	strcpy(tmp, oldpath);
        if (Filename == NULL)
        {
        	if(!getFilePath(tmp,EXT_GB|EXT_ZIP))
                 return 0;
        }
        else 
        {
             strcpy(tmp, Filename);
        }
	gb_init();
	
	if (getExtId(tmp) & EXT_ZIP){
		romsize = unzipfile(rom_image, MAX_ROM_SIZE, tmp, EXT_GB);
		if (!romsize)
			return -1;
	}
	else{
        fp = fopen(tmp, "rb");
        if(!fp)
            return -1;

		fseek (fp, 0, SEEK_END);
		fileSize = ftell (fp);
		rewind (fp);
		
		if (fileSize > MAX_ROM_SIZE)
			return -1;
		
		romsize = 0;
		while(romsize < fileSize)
		{
            bytes_read = fread(rom_image + romsize, 1, 0x200, fp);
			if(bytes_read == 0)
				break;
			romsize += bytes_read;
		}

		fclose(fp);
       
		if(romsize != fileSize)
			return -1;
	}
	
	strcpy(RomPath, tmp);
	char *p = strrchr(RomPath, '/');
	if(p)
		p++;
	else
		p = RomPath;
  
	strcpy(RomName, p);
	p = strrchr(RomName, '.');
	if(p!=NULL)
		*p = 0;
	
	ramsize = load_sram(sram_space, sizeof(sram_space));
	
	if (!gb_load_rom(rom_image, romsize, sram_space, ramsize))
		return -1;
	
	strcpy(oldpath, tmp);
	
	return 1;
}
#endif

void set_gb_type()
{
	if (rom_get_loaded()){
		switch(setting.gb_type){
		case 0:
			rom_get_info()->gb_type = org_gbtype;
			if(org_gbtype==1)
				lcd_set_mpal(setting.gb_palette);
			break;
		case 1:
			rom_get_info()->gb_type = 1;
			lcd_set_mpal(setting.gb_palette);
			break;
		case 2:
			if(org_sgb_mode){
				rom_get_info()->gb_type = 2;
			}
			else{
				rom_get_info()->gb_type = 1;
				lcd_set_mpal(setting.gb_palette);
			}
			break;
		case 3:
			if(org_gbtype<=2){
				rom_get_info()->gb_type = 1;
				lcd_set_mpal(setting.gb_palette);
			}else if(org_gbtype == 3){
				rom_get_info()->gb_type = 3;
			}
			break;
		case 4:
			if(org_gbtype<=2){
				rom_get_info()->gb_type = 1;
				lcd_set_mpal(setting.gb_palette);
			}else if(org_gbtype == 3){
				rom_get_info()->gb_type = 4;
			}
			break;
		}
		
		if(rom_get_info()->gb_type>=3 && org_gbtype==3)
			now_gb_mode = 3;
		else if(rom_get_info()->gb_type==2)
			now_gb_mode = 2;
		else
			now_gb_mode = 1;
		
		if(setting.sgb_border==0 && now_gb_mode!=2)
			sgb_mode = 0;
		else
			sgb_mode = org_sgb_mode;
	}
}

void configInit()
{
	setting.screensize = 0;
	setting.gb_palette = 0;
	setting.gb_type = 0;
	setting.sgb_border = 0;
	setting.lastpath[0] = 0;
}

void Reboot() {
#ifdef __wii__
    SYS_ResetSystem(SYS_RETURNTOMENU, 0, 0);
#else
#define SOFTRESET_ADR ((volatile u32*)0xCC003024)
    *SOFTRESET_ADR = 0x00000000;
#endif
}

void (*PSOReload)() = (void(*)())0x80001800;

void rin_menu(void)
{
	enum
	{
		SAVE_SRAM,
		SAVE_STATE,
		LOAD_STATE,
		SCREEN_SIZE,
		GB_PALETTE,
		GB_TYPE,
		SGB_BORDER,
		LOAD_ROM,
		RESET_EMU,
		RELOAD,
        RESET_CONSOLE,
		CONTINUE,
	};
	char msg[256]={0};
	static int sel=0;
	int x, y, ret, crs_count=0, bLoop=1;
	
	old_pad = 0;
	readpad();
	
	for(;;){
		readpad();
		if(new_pad){
			msg[0]=0;
			crs_count=0;
		}
		if(new_pad & PAD_BUTTON_A){
			crs_count=0;
			switch(sel)
			{
			case SAVE_SRAM:
				if (rom_get_loaded() && rom_has_battery())
					save_sram(get_sram(), rom_get_info()->ram_size);
				break;
			case SAVE_STATE:
				if (rom_get_loaded())
					save_sate();
				break;
			case LOAD_STATE:
				if (rom_get_loaded())
					load_state();
				break;
			case SCREEN_SIZE:
				setting.screensize++;
				if(setting.screensize>1)
					setting.screensize = 0;
				break;
			case GB_PALETTE:
				setting.gb_palette++;
				if(setting.gb_palette>=PAL_SGB)
					setting.gb_palette = 0;
				lcd_set_mpal(setting.gb_palette);
				break;
			case GB_TYPE:
				setting.gb_type++;
				if(setting.gb_type>4)
					setting.gb_type = 0;
				gb_reset();
				break;
			case SGB_BORDER:
				setting.sgb_border^=1;
				gb_reset();
				break;
			case LOAD_ROM:
				if (rom_get_loaded() && rom_has_battery())
					save_sram(get_sram(), rom_get_info()->ram_size);
				ret = load_rom(NULL);
				if(ret==1)
					bLoop = 0;
				else if(ret<0){
					while(!load_rom(NULL))
						;
					bLoop = 0;
				}
				break;
			case RESET_EMU:
				gb_reset();
				bLoop = 0;
				break;
			case RELOAD:				
				PSOReload();
				break;
            case RESET_CONSOLE:
                Reboot();
                break;
			case CONTINUE:
				bLoop = 0;
				break;
			}
		}else if(new_pad & PAD_BUTTON_B){
			bLoop = 0;
		}else if(new_pad & PAD_BUTTON_UP){
			if(sel!=0) sel--;
			else       sel=CONTINUE;
		}else if(new_pad & PAD_BUTTON_DOWN){
			if(sel!=CONTINUE)	sel++;
			else				sel=0;
		}else if(new_pad & PAD_BUTTON_LEFT){
			if(sel>LOAD_ROM)
				sel=LOAD_ROM;
			else if(sel>SCREEN_SIZE)
				sel=SCREEN_SIZE;
			else
				sel=0;
		}else if(new_pad & PAD_BUTTON_RIGHT){
			if(sel<SCREEN_SIZE)
				sel=SCREEN_SIZE;
			else if(sel<LOAD_ROM)
				sel=LOAD_ROM;
			else
				sel=CONTINUE;
		}
		
		if(!bLoop) break;
		if (crs_count++>=30) crs_count=0;
		
		VIDEO_ClearFrameBuffer (vmode, xfb[pg_drawframe], COLOR_BLACK);
		
		x = 35;
		y = 10;

		pgPrintf(x,y+=20,COLOR_WHITE,"SAVE SRAM");
		pgPrintf(x,y+=20,COLOR_WHITE,"SAVE STATE");
		pgPrintf(x,y+=20,COLOR_WHITE,"LOAD STATE");
		y+=20;
		pgPrintf(x,y+=20,COLOR_WHITE,"SCREEN SIZE   : %s",scr_names[setting.screensize]);
		pgPrintf(x,y+=20,COLOR_WHITE,"GB PALETTE    : %s",pal_names[setting.gb_palette]);
		pgPrintf(x,y+=20,COLOR_WHITE,"GB TYPE       : %s",gbtype_names[setting.gb_type]);
		pgPrintf(x,y+=20,COLOR_WHITE,"GBC+SGB BORDER: %s",setting.sgb_border?"ON":"OFF");
		y+=20;
		pgPrintf(x,y+=20,COLOR_WHITE,"Load New Game");
		pgPrintf(x,y+=20,COLOR_WHITE,"Reset Emulator");
		#ifdef __wii__
		pgPrintf(x,y+=20,COLOR_WHITE,"TP Reload");
        pgPrintf(x,y+=20,COLOR_WHITE,"Reset Wii");
		#else
		pgPrintf(x,y+=20,COLOR_WHITE,"PSO/SD Reload");
        pgPrintf(x,y+=20,COLOR_WHITE,"Reset GameCube");
		#endif
		pgPrintf(x,y+=20,COLOR_WHITE,"Play Game");
		
		if(crs_count < 15){
			y = sel;
			if(sel >= SCREEN_SIZE) y++;
			if(sel >= LOAD_ROM) y++;
			pgPrintf(x-15, y*20+30, COLOR_WHITE, ">");
		}
		
		VIDEO_SetNextFramebuffer(xfb[pg_drawframe]);
		VIDEO_Flush();
		VIDEO_WaitVSync();
		pg_showframe ^= 1;
		pg_drawframe ^= 1;
	}
	
	VIDEO_ClearFrameBuffer (vmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (vmode, xfb[1], COLOR_BLACK);
	gb_fill_vframe(0);
	usleep(5000);
	
	/*
	memset(&paddata, 0x00, sizeof(paddata));
	wavoutClear();
	*/
}
